@@ -5,7 +5,15 @@ from typing import *
 
 from cognite.client import utils
 from cognite.client._api_client import APIClient
-from cognite.client.data_classes import Asset, AssetFilter, AssetList, AssetUpdate, TimestampRange
+from cognite.client.data_classes import (
+    Asset,
+    AssetAggregate,
+    AssetFilter,
+    AssetList,
+    AssetUpdate,
+    LabelFilter,
+    TimestampRange,
+)
 from cognite.client.exceptions import CogniteAPIError
 
 
@@ -24,6 +32,9 @@ class AssetsAPI(APIClient):
         asset_subtree_ids: List[int] = None,
         asset_subtree_external_ids: List[str] = None,
         metadata: Dict[str, str] = None,
+        data_set_ids: List[int] = None,
+        data_set_external_ids: List[str] = None,
+        labels: LabelFilter = None,
         source: str = None,
         created_time: Union[Dict[str, Any], TimestampRange] = None,
         last_updated_time: Union[Dict[str, Any], TimestampRange] = None,
@@ -31,6 +42,7 @@ class AssetsAPI(APIClient):
         external_id_prefix: str = None,
         aggregated_properties: List[str] = None,
         limit: int = None,
+        partitions: int = None,
     ) -> Generator[Union[Asset, AssetList], None, None]:
         """Iterate over assets
 
@@ -46,6 +58,9 @@ class AssetsAPI(APIClient):
             asset_subtree_ids (List[int]): List of asset subtrees ids to filter on.
             asset_subtree_external_ids (List[str]): List of asset subtrees external ids to filter on.
             metadata (Dict[str, str]): Custom, application specific metadata. String key -> String value
+            data_set_ids (List[int]): Return only assets in the specified data sets with these ids.
+            data_set_external_ids (List[str]): Return only assets in the specified data sets with these external ids.
+            labels (LabelFilter): Return only the assets matching the specified label.
             source (str): The source of this asset
             created_time (Union[Dict[str, int], TimestampRange]):  Range between two timestamps. Possible keys are `min` and `max`, with values given as time stamps in ms.
             last_updated_time (Union[Dict[str, int], TimestampRange]):  Range between two timestamps. Possible keys are `min` and `max`, with values given as time stamps in ms.
@@ -53,6 +68,7 @@ class AssetsAPI(APIClient):
             external_id_prefix (str): Filter by this (case-sensitive) prefix for the external ID.
             aggregated_properties (List[str]): Set of aggregated properties to include.
             limit (int, optional): Maximum number of assets to return. Defaults to return all items.
+            partitions (int): Retrieve assets in parallel using this number of workers. Also requires `limit=None` to be passed.
 
         Yields:
             Union[Asset, AssetList]: yields Asset one by one if chunk is not specified, else AssetList objects.
@@ -64,6 +80,8 @@ class AssetsAPI(APIClient):
             root_ids = self._process_ids(root_ids, root_external_ids, wrap_ids=True)
         if asset_subtree_ids or asset_subtree_external_ids:
             asset_subtree_ids = self._process_ids(asset_subtree_ids, asset_subtree_external_ids, wrap_ids=True)
+        if data_set_ids or data_set_external_ids:
+            data_set_ids = self._process_ids(data_set_ids, data_set_external_ids, wrap_ids=True)
 
         filter = AssetFilter(
             name=name,
@@ -71,6 +89,8 @@ class AssetsAPI(APIClient):
             parent_external_ids=parent_external_ids,
             root_ids=root_ids,
             asset_subtree_ids=asset_subtree_ids,
+            data_set_ids=data_set_ids,
+            labels=labels,
             metadata=metadata,
             source=source,
             created_time=created_time,
@@ -78,11 +98,13 @@ class AssetsAPI(APIClient):
             root=root,
             external_id_prefix=external_id_prefix,
         ).dump(camel_case=True)
+
         return self._list_generator(
             method="POST",
             chunk_size=chunk_size,
             filter=filter,
             limit=limit,
+            partitions=partitions,
             other_params={"aggregatedProperties": aggregated_properties} if aggregated_properties else {},
         )
 
@@ -168,6 +190,9 @@ class AssetsAPI(APIClient):
         root_external_ids: List[str] = None,
         asset_subtree_ids: List[int] = None,
         asset_subtree_external_ids: List[str] = None,
+        data_set_ids: List[int] = None,
+        data_set_external_ids: List[str] = None,
+        labels: LabelFilter = None,
         metadata: Dict[str, str] = None,
         source: str = None,
         created_time: Union[Dict[str, Any], TimestampRange] = None,
@@ -188,6 +213,9 @@ class AssetsAPI(APIClient):
             root_external_ids (List[str], optional): List of root external ids to filter on.
             asset_subtree_ids (List[int]): List of asset subtrees ids to filter on.
             asset_subtree_external_ids (List[str]): List of asset subtrees external ids to filter on.
+            data_set_ids (List[int]): Return only assets in the specified data sets with these ids.
+            data_set_external_ids (List[str]): Return only assets in the specified data sets with these external ids.
+            labels (LabelFilter): Return only the assets matching the specified label filter.
             metadata (Dict[str, str]): Custom, application specific metadata. String key -> String value.
             source (str): The source of this asset.
             created_time (Union[Dict[str, int], TimestampRange]):  Range between two timestamps. Possible keys are `min` and `max`, with values given as time stamps in ms.
@@ -223,6 +251,14 @@ class AssetsAPI(APIClient):
                 >>> c = CogniteClient()
                 >>> for asset_list in c.assets(chunk_size=2500):
                 ...     asset_list # do something with the assets
+
+            Filter assets based on labels::
+
+                >>> from cognite.client import CogniteClient
+                >>> from cognite.client.data_classes import LabelFilter
+                >>> c = CogniteClient()
+                >>> my_label_filter = LabelFilter(contains_all=["PUMP", "VERIFIED"])
+                >>> asset_list = c.assets.list(labels=my_label_filter)
         """
         if aggregated_properties:
             aggregated_properties = [utils._auxiliary.to_camel_case(s) for s in aggregated_properties]
@@ -232,6 +268,8 @@ class AssetsAPI(APIClient):
             root_ids = self._process_ids(root_ids, root_external_ids, wrap_ids=True)
         if asset_subtree_ids or asset_subtree_external_ids:
             asset_subtree_ids = self._process_ids(asset_subtree_ids, asset_subtree_external_ids, wrap_ids=True)
+        if data_set_ids or data_set_external_ids:
+            data_set_ids = self._process_ids(data_set_ids, data_set_external_ids, wrap_ids=True)
 
         filter = AssetFilter(
             name=name,
@@ -239,6 +277,8 @@ class AssetsAPI(APIClient):
             parent_external_ids=parent_external_ids,
             root_ids=root_ids,
             asset_subtree_ids=asset_subtree_ids,
+            data_set_ids=data_set_ids,
+            labels=labels,
             metadata=metadata,
             source=source,
             created_time=created_time,
@@ -254,9 +294,28 @@ class AssetsAPI(APIClient):
             partitions=partitions,
         )
 
+    def aggregate(self, filter: Union[AssetFilter, Dict] = None) -> List[AssetAggregate]:
+        """`Aggregate assets <https://docs.cognite.com/api/v1/#operation/aggregateAssets>`_
+
+        Args:
+            filter (Union[AssetFilter, Dict]): Filter on assets filter with exact match
+
+        Returns:
+            List[AssetAggregate]: List of asset aggregates
+
+        Examples:
+
+            Aggregate assets:
+
+                >>> from cognite.client import CogniteClient
+                >>> c = CogniteClient()
+                >>> aggregate_by_prefix = c.assets.aggregate(filter={"external_id_prefix": "prefix"})
+        """
+        return self._aggregate(filter=filter, cls=AssetAggregate)
+
     def create(self, asset: Union[Asset, List[Asset]]) -> Union[Asset, AssetList]:
         """`Create one or more assets. <https://docs.cognite.com/api/v1/#operation/createAssets>`_
-        
+
         You can create an arbitrary number of assets, and the SDK will split the request into multiple requests.
         When specifying parent-child relation between assets using `parentExternalId` the link will be resvoled into an internal ID and stored as `parentId`.
 
@@ -275,6 +334,14 @@ class AssetsAPI(APIClient):
                 >>> c = CogniteClient()
                 >>> assets = [Asset(name="asset1"), Asset(name="asset2")]
                 >>> res = c.assets.create(assets)
+
+            Create asset with label::
+
+                >>> from cognite.client import CogniteClient
+                >>> from cognite.client.data_classes import Asset, Label
+                >>> c = CogniteClient()
+                >>> asset = Asset(name="my_pump", labels=[Label(external_id="PUMP")])
+                >>> res = c.assets.create(assets)
         """
         utils._auxiliary.assert_type(asset, "asset", [Asset, list])
         return self._create_multiple(asset)
@@ -298,7 +365,7 @@ class AssetsAPI(APIClient):
                 >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes import Asset
                 >>> c = CogniteClient()
-                >>> assets = [Asset(external_id="root"), Asset(external_id="child1", parent_external_id="root"), Asset(external_id="child2", parent_external_id="root")]
+                >>> assets = [Asset(external_id="root", name="root"), Asset(external_id="child1", parent_external_id="root", name="child1"), Asset(external_id="child2", parent_external_id="root", name="child2")]
                 >>> res = c.assets.create_hierarchy(assets)
         """
         utils._auxiliary.assert_type(assets, "assets", [list])
@@ -339,6 +406,7 @@ class AssetsAPI(APIClient):
 
     def update(self, item: Union[Asset, AssetUpdate, List[Union[Asset, AssetUpdate]]]) -> Union[Asset, AssetList]:
         """`Update one or more assets <https://docs.cognite.com/api/v1/#operation/updateAssets>`_
+        Currently, a full replacement of labels on an asset is not supported (only partial add/remove updates). See the example below on how to perform partial labels update.
 
         Args:
             item (Union[Asset, AssetUpdate, List[Union[Asset, AssetUpdate]]]): Asset(s) to update
@@ -363,6 +431,22 @@ class AssetsAPI(APIClient):
                 >>> c = CogniteClient()
                 >>> my_update = AssetUpdate(id=1).description.set("New description").metadata.add({"key": "value"})
                 >>> res = c.assets.update(my_update)
+
+            Attach labels to an asset::
+
+                >>> from cognite.client import CogniteClient
+                >>> from cognite.client.data_classes import AssetUpdate
+                >>> c = CogniteClient()
+                >>> my_update = AssetUpdate(id=1).labels.add(["PUMP", "VERIFIED"])
+                >>> res = c.assets.update(my_update)
+
+            Detach a single label from an asset::
+
+                >>> from cognite.client import CogniteClient
+                >>> from cognite.client.data_classes import AssetUpdate
+                >>> c = CogniteClient()
+                >>> my_update = AssetUpdate(id=1).labels.remove("PUMP")
+                >>> res = c.assets.update(my_update)
         """
         return self._update_multiple(items=item)
 
@@ -412,6 +496,13 @@ class AssetsAPI(APIClient):
                 >>> from cognite.client import CogniteClient
                 >>> c = CogniteClient()
                 >>> res = c.assets.search(name="xyz",filter={"parent_ids": [123,456],"source": "some source"})
+
+            Search for an asset with an attached label:
+
+                >>> from cognite.client import CogniteClient
+                >>> c = CogniteClient()
+                >>> my_label_filter = LabelFilter(contains_all=["PUMP"])
+                >>> res = c.assets.search(name="xyz",filter=AssetFilter(labels=my_label_filter))
         """
         return self._search(
             search={"name": name, "description": description, "query": query}, filter=filter, limit=limit
